﻿using System;
using System.Collections.Generic;
using System.Text;
using Library;
using Kuzushi.Resources;
using System.Drawing;
using Tsukikage.GameSDK.GSystem;
using Tsukikage.GameSDK.Direct3D;
using Tsukikage.GameSDK.DirectInput;

namespace Kuzushi.Engine
{
    /// <summary>
    /// すべてのゲーム用オブジェクトの基底
    /// 今回は要らなかったかもしれない。
    /// </summary>
    public abstract class GObject
    {
        /// <summary>
        /// オブジェクトの位置
        /// </summary>
        Vector2 location;

        /// <summary>
        /// 当たり判定矩形
        /// </summary>
        Rect hitTestRect;

        /// <summary>
        /// オブジェクトの位置
        /// </summary>
        public Vector2 Location
        {
            get { return location; }
            set { location = value; }
        }

        /// <summary>
        /// オブジェクトの位置
        /// </summary>
        public int X
        {
            get { return location.X; }
            set { location = new Vector2(value, Y); }
        }

        /// <summary>
        /// オブジェクトの位置
        /// </summary>
        public int Y
        {
            get { return location.Y; }
            set { location = new Vector2(X, value); }
        }

        /// <summary>
        /// 当たり判定矩形
        /// </summary>
        public Rect HittestRect
        {
            get { return hitTestRect; }
            set { hitTestRect = value; }
        }

        /// <summary>
        /// 指定したオブジェクトとこのオブジェクトが衝突しているかどうかを判定
        /// </summary>
        /// <param name="obj">対象のオブジェクト</param>
        /// <returns>衝突している場合true</returns>
        public bool IntersectWith(GObject obj)
        {
            return hitTestRect.Offset(location).IntersectWith(obj.hitTestRect.Offset(obj.location));
        }

        /// <summary>
        /// 内部処理
        /// </summary>
        public abstract void Internal();

        /// <summary>
        /// 描画処理
        /// </summary>
        public abstract void Draw();
    }

    /// <summary>
    /// ボールだよー
    /// </summary>
    public class GBall : GObject
    {
        /// <summary>
        /// 私を包み込んでくれるエンジン
        /// </summary>
        Engine engine;

        /// <summary>
        /// ボールの速度
        /// </summary>
        public double SpdX, SpdY;

        /// <summary>
        /// 座標をDouble型でも持つ
        /// </summary>
        public double PosX, PosY;

        public int Radius;

        /// <summary>
        /// ボールの初期設定
        /// </summary>
        /// <param name="engine">ボールはエンジンにアクセスする必要がありそう</param>
        public GBall(Engine engine)
        {
            this.engine = engine;

            this.SpdX = 4;
            this.SpdY = 4;
            this.PosX = this.PosY = 0;

            this.Radius = 10;
            this.HittestRect = new Rect(-Radius, -Radius, Radius * 2, Radius * 2);

        }

        /// <summary>
        /// 内部処理
        /// </summary>
        public override void Internal()
        {
            PosX += SpdX;
            PosY += SpdY;
            Location = new Vector2((int)PosX, (int)PosY);

            // ラケットとぶつかっている
            if (engine.Racket.IntersectWith(this))
            {
                SpdY = -Math.Abs(SpdY);
                Sounds.Cursor.PlayOnce();
            }

            // すべてのブロックに対して、デリゲート { } で指定した動作を行う 
            engine.ForeachBlocks(delegate(GBlock block)
                {
                    // 衝突しているか円判定するよ
                    double dX = block.Location.X - PosX; // 距離X
                    double dY = block.Location.Y - PosY; // 距離Y
                    double sR = block.Radius + Radius; // 半径の和

                    // 三平方の定理より、中心間の距離＜半径の和 であるとき円は交差している。
                    if ( (dX * dX) + (dY * dY) < (sR * sR))
                    {
                        // ブロックを取り除く
                        engine.RemoveBlock(block);

                        // 点数がもらえる
                        engine.AddScore(200);

                        // 音を出す
                        Sounds.Cursor.PlayOnce();

                        // **** ここからボール反射方向の計算

                        // ボールとブロックの中心距離
                        double distance = Math.Sqrt(dX * dX + dY * dY);

                        // 反射方向の計算 : 接線方向ベクトルS 法線方向ベクトルN
                        double normNX = dX / distance;
                        double normNY = dY / distance;
                        double normSX = dY / distance;
                        double normSY = -dX / distance;

                        // 法線方向のスピードと接線方向のスピードに分解(射影)
                        double spdN = SpdX * normNX + SpdY * normNY;
                        double spdS = SpdX * normSX + SpdY * normSY;

                        // 法線方向成分を反転して X-Y空間へ戻。
                        // 新しいボール速度が決定！
                        SpdX = -spdN * normNX + spdS * normSX;
                        SpdY = -spdN * normNY + spdS * normSY;

                        // **** ここまでボール反射方向の計算。
                    }
                });

        }

        /// <summary>
        /// 描画処理
        /// </summary>
        public override void Draw()
        {
            D3DChara chip = Textures.Circle.CreateChara(new Rectangle(0, 0, 256, 256), new Point(128, 128));
            chip.Draw((Point)Location, Radius / 128.0, Radius / 128.0, 0, Argb.White);
        }
    }

    /// <summary>
    /// ラケットだよー
    /// </summary>
    public class GRacket : GObject
    {
        /// <summary>
        /// 初期化処理
        /// </summary>
        public GRacket()
        {
            X = 320;
            Y = 380;
            HittestRect = new Rect(-40, -10, 80, 20);
        }

        /// <summary>
        /// 内部処理
        /// </summary>
        public override void Internal()
        {
            KeyStat[] buttons = GSDK.DInput.ButtonsState;

            // 上下左右と覚える
            if (buttons[0].Pressed) { }
            if (buttons[1].Pressed) { }
            if (buttons[2].Pressed) { this.X -= 10; }
            if (buttons[3].Pressed) { this.X += 10; }
        }

        /// <summary>
        /// 描画処理
        /// </summary>
        public override void Draw()
        {
            GSDK.D3D.DrawFilledRectangle((Rectangle)HittestRect.Offset(Location), Argb.Red);
            Fonts.Uzura18pt.DrawText(new Point(200, 200), "" + X, Argb.Yellow);
        }
    }


    /// <summary>
    /// ブロックだよー
    /// </summary>
    public class GBlock : GObject
    {
        /// <summary>
        /// ブロック半径
        /// </summary>
        public double Radius;

        /// <summary>
        /// ブロック色
        /// </summary>
        Argb color;

        /// <summary>
        /// 円キャラ
        /// </summary>
        D3DChara chara;

        /// <summary>
        /// コンストラクタ(初期化)
        /// </summary>
        /// <param name="color"></param>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="r"></param>
        public GBlock(Argb color, int x, int y, int r)
        {
            this.color = color;
            this.Location = new Vector2(x, y);
            this.Radius = r;

            this.chara = Textures.Circle.CreateChara(new Rectangle(0, 0, 256, 256), new Point(128, 128));
        }

        /// <summary>
        /// 内部処理
        /// </summary>
        public override void Internal()
        {
            // 毎フレーム下に2px動く
            Y += 3;
        }

        /// <summary>
        /// 描画処理
        /// </summary>
        public override void Draw()
        {
            // Locationの座標に
            // 半径128の円を radius/128倍して 0回転させて
            // color を乗算して描画
            chara.Draw((Point)Location, Radius / 128, Radius / 128, 0, color);
        }
    }
}
